/*
 * dlgloadf.c --- load font dialog
 * 
 * Copyright (c) 2019 Mariano Alvarez Fernandez (malfer@telefonica.net)
 *
 * This file is part of FEdit, a visual font editor for the MGRX font format
 *
 * FEdit is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FEdit is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FEdit. If not, see <http://www.gnu.org/licenses/>.
 *
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <grgui.h>
#include <mgrxcolr.h>
#include "fedit.h"

#define COMMAND_OK           1
#define COMMAND_CANCEL       2

GUIDialog *floaddlg = NULL;
GUIGroup *floadgo = NULL;

int process_floaddlg_event(void *udata, GrEvent *ev)
{
    GUIDialog *d = (GUIDialog *)udata;
    GUIGroup *go = (GUIGroup *)(d->exdata);

    if (ev->type == GREV_COMMAND) {
        switch (ev->p1) {
            case COMMAND_OK :
                d->result = 1;
               return -1;
            case COMMAND_CANCEL :
                d->result = -1;
                return -1;
        }
    }

    return GUIGroupProcessEvent(go, ev);
}

int FLoadDlgCreate(void)
{
    if (floaddlg) return 1; // dialog already created

    floadgo = GUIGroupCreate(4, 0, 0);
    if (floadgo == NULL) return 0;
    GUIObjectSetLabel(&(floadgo->o[0]), 0,   0,   0, 160, 30, GrNOCOLOR, WEBC_WHITE, "Font name");
    GUIObjectSetEntry(&(floadgo->o[1]), 1, 160,   0, 160, 30, WEBC_WHITE, WEBC_BLACK, MAXLEN_FNAME, "");
    GUIObjectSetButton(&(floadgo->o[2]), 2,  10, 50, 140, 40, WEBC_GREEN, WEBC_WHITE, "Ok", COMMAND_OK, 0, 0);
    GUIObjectSetButton(&(floadgo->o[3]), 3, 170, 50, 140, 40, WEBC_GREEN, WEBC_WHITE, "Cancel", COMMAND_CANCEL, 0, 0);
    GUIGroupSetSelected(floadgo, 1, 0);

    floaddlg = GUIGroupDialogCreate("Load Font",
                                    floadgo, process_floaddlg_event);
    if (floaddlg == NULL) {
        GUIGroupDestroy(floadgo);
        floadgo = NULL;
        return 0;
    }

    return 1;
}

void FLoadDlgDestroy(void)
{
    if (floaddlg) {
        GUIDialogDestroy(floaddlg);
        floaddlg = NULL;
    }
    if (floadgo) {
        GUIGroupDestroy(floadgo);
        floadgo = NULL;
    }
}

void FLoadDlgSetData(char *fname)
{
    if (!floaddlg) return;

    GUIGroupSetText(floadgo, 1, fname);
}

void FLoadDlgGetData(char *fname)
{
    char *s;

    if (!floaddlg) return;

    s = GUIGroupGetText(floadgo, 1, GrGetChrtypeForUserEncoding());
    strcpy(fname, s);
    free(s);
}

int FLoadDlgRun(void)
{
    if (!floaddlg) return -1; 

    floaddlg->result = -1;
    GUIDialogRun(floaddlg);
    return floaddlg->result;
}

