#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <mgrx.h>

#include "hershey.h"

HersheyFont * LoadHersheyFont(char *fname, int minchar, int maxchars, int *err)
{
    FILE *f = NULL;
    HersheyFont *hf = NULL;
    char buf[401];
    int len, nchars, np, minx, maxx, width, i, xh, yh;
    char nps[4], *p;
    //int nline;
    
    *err = 0;
    f = fopen(fname,"r");
    if (f == NULL) {
        *err = 1;
        return NULL;
    }

    nchars = 0;
    //nline = 0;
    while(1) {
        //nline++;
        if (fgets(buf, 400, f) == NULL) break;
        if (buf[0] == '#') continue;
        len = strlen(buf);
        if ((len > 0) && (buf[len-1] == '\n')) {
            buf[len-1] = '\0';
            len--;
        }
        if (len < 10) {
            *err = 2;
            goto error;
        }
        nps[0] = buf[5];
        nps[1] = buf[6];
        nps[2] = buf[7];
        nps[3] = '\0';
        np = atoi(nps);
        if (len < np*2+8) {
            *err = 3;
            //printf("nline %d\n", nline);
            goto error;
        }
        nchars++;
    }

    if (nchars > maxchars) nchars = maxchars;

    hf= (HersheyFont *) calloc(1, sizeof(HersheyFont));
    if (hf == NULL) {
        *err = 4;
        return NULL;
    }

    hf->maxy = -1000;
    hf->miny = 1000;
    hf->maxwidth = 0;
    hf->minchar = minchar;
    hf->numchars = nchars;
    hf->glyphdef = (char **)calloc(nchars, sizeof(char *));
    if (hf->glyphdef == NULL) {
        *err = 5;
        return NULL;
    }

    rewind(f);
    nchars = 0;
    while(1) {
        if (fgets(buf, 400, f) == NULL) break;
        if (nchars >= hf->numchars) break;
        if (buf[0] == '#') continue;
        len = strlen(buf);
        if ((len > 0) && (buf[len-1] == '\n')) {
            buf[len-1] = '\0';
            len--;
        }
        nps[0] = buf[5];
        nps[1] = buf[6];
        nps[2] = buf[7];
        nps[3] = '\0';
        np = atoi(nps);
        hf->glyphdef[nchars] = strdup(buf);
        if (hf->glyphdef[nchars] == NULL) {
            *err = 6;
            goto error;
        }
        minx = buf[8] - 'R';
        maxx = buf[9] - 'R';
        p = &(buf[10]);
        for(i=0; i <np-1; i++) {
            if (*p != ' ') {
                xh = *p - 'R';
                yh = *(p+1) - 'R';
                if (xh > maxx) maxx = xh;
                if (xh < minx) minx = xh;
                if (yh > hf->maxy) hf->maxy = yh;
                if (yh < hf->miny) hf->miny = yh;
            }
            p += 2;
        }
        width = maxx - minx + 1;
        if (width > hf->maxwidth) hf->maxwidth = width;
        nchars++;
    }

    fclose(f);
    
    return hf;

error:
    if (f) fclose(f);
    if (hf) {
        if (hf->glyphdef) {
            for (i=0; i<hf->numchars; i++) {
                if (hf->glyphdef[i]) free(hf->glyphdef[i]);
            }
            free(hf->glyphdef);
        }
        free(hf);
    }
    return NULL;
}

int DrawHersheyFontGlyph(HersheyFont *hf, int glyph, int x, int y,
                         double zoom, int thick, GrColor c)
{
    int xh, yh, xhold, yhold, np, move, i;
    int min, max, width;
    char *hers, nps[4], *p;
    GrLineOption lineop;

    lineop.lno_color = c;
    lineop.lno_width = thick;
    lineop.lno_pattlen = 0;
    lineop.lno_dashpat = (unsigned char *)"\0";

    if (hf == NULL) return 0;
    if (glyph < hf->minchar) return 0;
    glyph -= hf->minchar;
    if (glyph >= hf->numchars) return 0;

    hers = hf->glyphdef[glyph];
    if (hers == NULL) return 0;
    
    nps[0] = hers[5];
    nps[1] = hers[6];
    nps[2] = hers[7];
    nps[3] = '\0';

    np = atoi(nps);

    min = hers[8] - 'R';
    max = hers[9] - 'R';

    p = &(hers[10]);
    for(i=0; i <np-1; i++) {
        if (*p != ' ') {
            xh = *p - 'R';
            if (xh > max) max = xh;
            if (xh < min) min = xh;
        }
        p += 2;
    }
    width = (max - min) * zoom  + 1;

    xhold = 0;
    yhold = 0;
    move=1;

    p = &(hers[10]);
    for(i=0; i <np-1; i++) {
        if (*p == ' ' && *(p+1) == 'R') {
            move = 1;
            p += 2;
            continue;
        }
        xh = *p - 'R' - min;
        yh = *(p+1) - 'R' - hf->miny;
        if (!move) {
            //GrLine(xhold*zoom+x, yhold*zoom+y, xh*zoom+x, yh*zoom+y, c);
            GrCustomLine(xhold*zoom+x, yhold*zoom+y, xh*zoom+x, yh*zoom+y, &lineop);
        }
        xhold = xh;
        yhold = yh;
        p += 2;
        move = 0;
    }
    
    return width;
}

void DrawHersheyFontString(HersheyFont *hf, char *s, int xorg, int yorg,
                           double zoom, int thick, GrColor c)
{
    int len, i;
    int width;

    len = strlen(s);
    
    for (i=0; i<len; i++) {
        width = DrawHersheyFontGlyph(hf, s[i], xorg, yorg, zoom, thick, c);
        xorg += width;
    }
}

void GetHersheyFontBox(HersheyFont *hf, double zoom, int *maxh, int *maxw)
{
    *maxh = (hf->maxy - hf->miny + 1) * zoom;
    *maxw = hf->maxwidth * zoom;
}

GrFont * GenGrxHersheyFont(HersheyFont *hf, char *name, char *family,
                           double zoom, int thick)
{
    GrFont *f = NULL;
    GrContext *wctx = NULL;
    GrContext grcaux;
    int nb, i, j, w;
    int bmprowsize, bmpbox, bmprowchar;
    long sumwith;
    char *boxptr, *pb, *chrptr, *pc;

    nb = sizeof(GrFont) + ((hf->numchars - 1) * sizeof(GrFontChrInfo));
    f = malloc(nb);
    if (f == NULL) goto error;
    memset(f, 0, nb);
    f->h.name = strdup(name);
    f->h.family = strdup(family);
    if ((f->h.name == NULL) || (f->h.family == NULL)) goto error;
    f->h.proportional = 1;
    f->h.scalable = 0;
    f->h.preloaded = 0;
    f->h.modified = 0;
    //f->h.width = do later
    f->h.height = (hf->maxy - hf->miny + 1) * zoom;
    f->h.baseline = f->h.height - 2;  // needs calcs
    f->h.ulpos = f->h.height - 1;     // needs calcs
    f->h.ulheight = 1;                // needs calcs
    f->h.minchar = hf->minchar;
    f->h.numchars = hf->numchars;
    f->h.encoding = 0;
    //f->minwidth = do later
    f->maxwidth = hf->maxwidth * zoom;

    bmprowsize = (f->maxwidth + 7) / 8;
    bmpbox = bmprowsize * f->h.height;
    f->bitmap = calloc(bmpbox, f->h.numchars); // is bigger than necesary, but...
    if (f->bitmap == NULL) goto error;

    wctx = GrCreateFrameContext(GR_frameRAM1, bmprowsize*8, f->h.height,
                                NULL, NULL);
    if (wctx == NULL) goto error;
    GrSaveContext(&grcaux);
    GrSetContext(wctx);
    boxptr = wctx->gc_baseaddr[0];
    nb = 0;
    sumwith = 0;
    f->minwidth = f->maxwidth;
    for (i=0; i<f->h.numchars; i++) {
        GrClearContext(GrBlack());
        w = DrawHersheyFontGlyph(hf, hf->minchar+i, 0, 0, zoom, thick, GrWhite());
        f->chrinfo[i].width  = w;
        f->chrinfo[i].offset = nb;
        bmprowchar = (w + 7) / 8;
        chrptr = f->bitmap + nb;
        for (j=0; j<f->h.height; j++) {
            pc = chrptr + (bmprowchar * j);
            pb = boxptr + (wctx->gc_lineoffset * j);
            memcpy(pc, pb, bmprowchar);
        }
        nb += bmprowchar * f->h.height;
        if (w < f->minwidth) f->minwidth = w;
        sumwith += w;
    }
    f->h.width = sumwith / f->h.numchars;
    GrSetContext(&grcaux);
    GrDestroyContext(wctx);

    return f;

    error:
    if (f != NULL) {
        if (f->h.name) free(f->h.name);
        if (f->h.family) free(f->h.family);
        if (f->bitmap) free(f->bitmap);
        free(f);
    }
    return NULL;
}

void UnloadHersheyFont(HersheyFont *hf)
{
    int i;

    if (hf) {
        for (i=0; i< hf->numchars; i++) {
            if (hf->glyphdef[i]) free(hf->glyphdef[i]);
        }
        free(hf->glyphdef);
        free(hf);
    }
}
